#include "stm32f4xx.h"
#include "stdio.h"
#define LED0 (1<<12)
#define LED1 (1<<13)
#define LED2 (1<<14)
#define LED3 (1<<15)
/* Macros for word accesses */
#define HW32_REG(ADDRESS)  (*((volatile unsigned long  *)(ADDRESS)))
/* Use Breakpoint to stop when error is detected (KEIL MDK specific intrinsic) */
/* it can be changed to while(1) if needed */
#define stop_cpu   __breakpoint(0)
void LED_initize(void); // Initialize LED
void task0(void);       // Toggle LED0
void task1(void);       // Toggle LED1
void task2(void);       // Toggle LED2
void task3(void);       // Toggle LED3
void __svc(0x00) os_start(void); // OS initialization
void SVC_Handler_C(unsigned int * svc_args);
void HardFault_Handler_C(unsigned int * svc_args);

// Event to tasks
volatile uint32_t systick_count=0;
// Stack for each task (8Kbytes each - 1024 x 8 bytes)
long long task0_stack[1024], task1_stack[1024], task2_stack[1024], task3_stack[1024];
// Data use by OS
uint32_t  curr_task=0;     // Current task
uint32_t  next_task=1;     // Next task
uint32_t  PSP_array[4];    // Process Stack Pointer for each task
uint32_t  svc_exc_return;  // EXC_RETURN use by SVC

// -------------------------------------------------------------------------
// Start of main program
int main(void)
{
  SCB->CCR |= SCB_CCR_STKALIGN_Msk; // Enable double word stack alignment 
  //(recommended in Cortex-M3 r1p1, default in Cortex-M3 r2px and Cortex-M4)

  puts("Simple context switching demo");
  LED_initize(); 
  os_start();
  
  while(1){		
    stop_cpu;// Should not be here
    };
}
// -------------------------------------------------------------------------
void task0(void) // Toggle LED #0
{ while (1) {
  if (systick_count & 0x80) {GPIOD->BSRRL = LED0;} // Set   LED 0
  else                      {GPIOD->BSRRH = LED0;} // Clear LED 0
  };
}
// -------------------------------------------------------------------------
void task1(void) // Toggle LED #1
{ while (1) {
  if (systick_count & 0x100){GPIOD->BSRRL = LED1;} // Set   LED 1
  else                      {GPIOD->BSRRH = LED1;} // Clear LED 1
  };
}
// -------------------------------------------------------------------------
void task2(void) // Toggle LED #2
{ while (1) {		
  if (systick_count & 0x200){GPIOD->BSRRL = LED2;} // Set   LED 2
  else                      {GPIOD->BSRRH = LED2;} // Clear LED 2
  };
}
// -------------------------------------------------------------------------
void task3(void) // Toggle LED #3
{ while (1) {
  if (systick_count & 0x400){GPIOD->BSRRL = LED3;} // Set   LED 3
  else                      {GPIOD->BSRRH = LED3;} // Clear LED 3
  };
}	
// -------------------------------------------------------------------------
__asm void SVC_Handler(void)
{
  TST    LR, #4   // Extract stack frame location
  ITE    EQ
  MRSEQ  R0, MSP
  MRSNE  R0, PSP
  LDR    R1,=__cpp(&svc_exc_return) // Save current EXC_RETURN
  STR    LR,[R1]	
  BL     __cpp(SVC_Handler_C)       // Run C part of SVC_Handler
  LDR    R1,=__cpp(&svc_exc_return) // Load new EXC_RETURN
  LDR    LR,[R1]
  BX     LR
  ALIGN  4
}
void SVC_Handler_C(unsigned int * svc_args)
{
  uint8_t svc_number;	
  svc_number = ((char *) svc_args[6])[-2]; // Memory[(Stacked PC)-2]
  switch(svc_number) {
    case (0): // OS init
      // Starting the task scheduler
      // Create stack frame for task0
      PSP_array[0] = ((unsigned int) task0_stack) + (sizeof task0_stack) - 18*4;
      HW32_REG((PSP_array[0] + (16<<2))) = (unsigned long) task0; // initial Program Counter
      HW32_REG((PSP_array[0] + (17<<2))) = 0x01000000;            // initial xPSR
      HW32_REG((PSP_array[0]          )) = 0xFFFFFFFDUL;          // initial EXC_RETURN
      HW32_REG((PSP_array[0] + ( 1<<2))) = 0x3;// initial CONTROL : unprivileged, PSP, no FP

      // Create stack frame for task1
      PSP_array[1] = ((unsigned int) task1_stack) + (sizeof task1_stack) - 18*4;
      HW32_REG((PSP_array[1] + (16<<2))) = (unsigned long) task1; // initial Program Counter
      HW32_REG((PSP_array[1] + (17<<2))) = 0x01000000;            // initial xPSR
      HW32_REG((PSP_array[1]          )) = 0xFFFFFFFDUL;          // initial EXC_RETURN
      HW32_REG((PSP_array[1] + ( 1<<2))) = 0x3;// initial CONTROL : unprivileged, PSP, no FP

      // Create stack frame for task2
      PSP_array[2] = ((unsigned int) task2_stack) + (sizeof task2_stack) - 18*4;
      HW32_REG((PSP_array[2] + (16<<2))) = (unsigned long) task2; // initial Program Counter
      HW32_REG((PSP_array[2] + (17<<2))) = 0x01000000;            // initial xPSR
      HW32_REG((PSP_array[2]          )) = 0xFFFFFFFDUL;          // initial EXC_RETURN
      HW32_REG((PSP_array[2] + ( 1<<2))) = 0x3;// initial CONTROL : unprivileged, PSP, no FP

      // Create stack frame for task3
      PSP_array[3] = ((unsigned int) task3_stack) + (sizeof task3_stack) - 18*4;
      HW32_REG((PSP_array[3] + (16<<2))) = (unsigned long) task3; // initial Program Counter
      HW32_REG((PSP_array[3] + (17<<2))) = 0x01000000;            // initial xPSR
      HW32_REG((PSP_array[3]          )) = 0xFFFFFFFDUL;          // initial EXC_RETURN
      HW32_REG((PSP_array[3] + ( 1<<2))) = 0x3;// initial CONTROL : unprivileged, PSP, no FP

      curr_task = 0; // Switch to task #0 (Current task)
      svc_exc_return = HW32_REG((PSP_array[curr_task])); // Return to thread with PSP
      __set_PSP((PSP_array[curr_task] + 10*4));  // Set PSP to @R0 of task 0 exception stack frame
      NVIC_SetPriority(PendSV_IRQn, 0xFF); // Set PendSV to lowest possible priority
      SysTick_Config(168000);              // 1000 Hz SysTick interrupt on 168MHz core clock
      __set_CONTROL(0x3);                  // Switch to use Process Stack, unprivileged state
      __ISB();       // Execute ISB after changing CONTROL (architectural recommendation)
      break;
    default:
      puts ("ERROR: Unknown SVC service number");
      printf("- SVC number 0x%x\n", svc_number);
      stop_cpu;
      break;
  } // end switch
}	

// -------------------------------------------------------------------------
__asm void PendSV_Handler(void)
{ // Context switching code
  // Simple version - assume No floating point support
  // -------------------------
  // Save current context
  MRS      R0, PSP     // Get current process stack pointer value
  TST      LR, #0x10   // Test bit 4. If zero, need to stack floating point regs
  IT       EQ
  VSTMDBEQ R0!, {S16-S31} // Save floating point registers
  MOV      R2, LR
  MRS      R3, CONTROL
  STMDB    R0!,{R2-R11}// Save LR,CONTROL and R4 to R11 in task stack (10 regs)
  LDR      R1,=__cpp(&curr_task)
  LDR      R2,[R1]     // Get current task ID
  LDR      R3,=__cpp(&PSP_array)
  STR      R0,[R3, R2, LSL #2] // Save PSP value into PSP_array
  // -------------------------
  // Load next context
  LDR      R4,=__cpp(&next_task)
  LDR      R4,[R4]     // Get next task ID
  STR      R4,[R1]     // Set curr_task = next_task
  LDR      R0,[R3, R4, LSL #2] // Load PSP value from PSP_array
  LDMIA    R0!,{R2-R11}// Load LR, CONTROL and R4 to R11 from task stack (10 regs)
  MOV      LR, R2
  MSR      CONTROL, R3
  TST      LR, #0x10   // Test bit 4. If zero, need to unstack floating point regs
  IT       EQ
  VLDMIAEQ R0!, {S16-S31} // Save floating point registers
  MSR      PSP, R0     // Set PSP to next task
  BX       LR          // Return
  ALIGN  4
}
// -------------------------------------------------------------------------
void SysTick_Handler(void) // 1KHz
{ // Increment systick counter for LED blinking
  systick_count++;
  // Simple task round robin scheduler
  switch(curr_task) {
    case(0): next_task=1; break;
    case(1): next_task=2; break;
    case(2): next_task=3; break;
    case(3): next_task=0; break;
    default: next_task=0;
      printf("ERROR:curr_task = %x\n", curr_task);
      stop_cpu;
      break; // Should not be here
    }
  if (curr_task!=next_task){ // Context switching needed
    SCB->ICSR |= SCB_ICSR_PENDSVSET_Msk; // Set PendSV to pending
    }
  return;	
}
// -------------------------------------------------------------------------
__asm void HardFault_Handler(void) // Report HardFault when things goes wrong
{
  TST    LR, #4
  ITE    EQ
  MRSEQ  R0, MSP
  MRSNE  R0, PSP
  B      __cpp(HardFault_Handler_C)
}	

void HardFault_Handler_C(unsigned int * svc_args)
{
  puts("[HardFault]");
  printf ("curr_task = %d\n", curr_task);
  printf ("next_task = %d\n", next_task);
  printf ("PSP #0 = %x\n", PSP_array[0]);
  printf ("PSP #1 = %x\n", PSP_array[1]);
  printf ("PSP #2 = %x\n", PSP_array[2]);
  printf ("PSP #3 = %x\n", PSP_array[3]);
  printf ("Stacked PC = %x\n", svc_args[6]);
  stop_cpu;	
}
// -------------------------------------------------------------------------
void LED_initize(void)
{
  // Configure LED outputs
  RCC->AHB1ENR |= RCC_AHB1ENR_GPIODEN; // Enable Port D clock
  // Set pin 12, 13, 14, 15 as general purpose output mode (pull-push) 
  GPIOD->MODER |= (GPIO_MODER_MODER12_0 |
                   GPIO_MODER_MODER13_0 |
                   GPIO_MODER_MODER14_0 |
                   GPIO_MODER_MODER15_0 ) ;
  // GPIOD->OTYPER  |= 0; // No need to change - use pull-push output
  GPIOD->OSPEEDR |= (GPIO_OSPEEDER_OSPEEDR12 |
                    GPIO_OSPEEDER_OSPEEDR13 |
                    GPIO_OSPEEDER_OSPEEDR14 |
                    GPIO_OSPEEDER_OSPEEDR15 );
  GPIOD->PUPDR = 0; // No pull up , no pull down  
  return;	
}
// -------------------------------------------------------------------------
